@@ -1,6 +1,7 @@
 #ifndef SF_INTERNAL_H
 #define SF_INTERNAL_H
 #include <Rcpp.h>
+#include <Rversion.h>
 #include <iostream>
 #include <string>
 #include <vector>
@@ -89,10 +90,15 @@ enum class cetype_t_ext : uint8_t {
 };
 
 // defn.h
-#define ASCII_MASK (1<<6)
-//#define IS_ASCII(x) ((x)->sxpinfo.gp & ASCII_MASK)
+// #define ASCII_MASK (1<<6)
+// #define IS_ASCII(x) ((x)->sxpinfo.gp & ASCII_MASK)
+// LEVELS is no longer public API
 inline bool IS_ASCII(SEXP x) {
-  return LEVELS(x) & ASCII_MASK;
+#if (R_VERSION >= R_Version(4, 5, 0))
+  return Rf_charIsASCII(x);
+#else
+  return checkAscii(CHAR(x), Rf_xlength(x));
+#endif
 }
 
 
@@ -130,6 +136,7 @@ inline cetype_t choose_enc(cetype_t x, cetype_t y, cetype_t z) {
 struct sfstring {
   std::string sdata;
   cetype_t_ext encoding;
+  sfstring(const char * ptr, int len, cetype_t_ext enc) : sdata(ptr, len), encoding(enc) {}
   sfstring(std::string x, cetype_t enc) : sdata(x) {
     if(checkAscii(sdata.c_str(), sdata.size())) {
       encoding = cetype_t_ext::CE_ASCII; // to keep the same as R
@@ -137,17 +144,14 @@ struct sfstring {
       encoding = static_cast<cetype_t_ext>(enc);
     }
   }
-  sfstring(const char * ptr, cetype_t enc) {
-    size_t len = strlen(ptr);
-    sdata = std::string(ptr);
-    if(checkAscii(ptr, len)) {
+  sfstring(const char * ptr, cetype_t enc) : sdata(ptr) {
+    if(checkAscii(sdata.data(), sdata.size())) {
       encoding = cetype_t_ext::CE_ASCII; // to keep the same as R
     } else {
       encoding = static_cast<cetype_t_ext>(enc);
     }
   }
-  sfstring(const char * ptr, int len, cetype_t enc) {
-    sdata = std::string(ptr, len);
+  sfstring(const char * ptr, int len, cetype_t enc) : sdata(ptr, len) {
     if(checkAscii(ptr, len)) {
       encoding = cetype_t_ext::CE_ASCII; // to keep the same as R
     } else {
@@ -211,6 +215,23 @@ enum class rstring_type : uint8_t {
 
 rstring_type get_rstring_type_internal(SEXP obj) {
   if(TYPEOF(obj) != STRSXP) throw std::runtime_error("Object not an Character Vector");
+#if R_VERSION >= R_Version(4, 6, 0)
+    SEXP class_name = R_altrep_class_name(obj); // R_NilValue if not ALTREP
+    if(class_name == R_NilValue) {
+      return rstring_type::NORMAL;
+    } else {
+      std::string classname = std::string(CHAR(PRINTNAME(class_name)));
+      if(classname == "__sf_vec__") {
+        if(DATAPTR_OR_NULL(obj) == nullptr) {
+          return rstring_type::SF_VEC;
+        } else {
+          return rstring_type::SF_VEC_MATERIALIZED;
+        }
+      } else {
+        return rstring_type::OTHER_ALT_REP;
+      }
+    }
+#else
   if(ALTREP(obj)) {
     SEXP pclass = ATTRIB(ALTREP_CLASS(obj));
     std::string classname = std::string(CHAR(PRINTNAME(CAR(pclass))));
@@ -226,13 +247,14 @@ rstring_type get_rstring_type_internal(SEXP obj) {
   } else {
     return rstring_type::NORMAL;
   }
+#endif
 }
 
 class RStringIndexer {
 private:
   size_t len;
   rstring_type type;
-  void * dptr; // should we use std::variant?
+  void * dptr;
 public:
   struct rstring_info {
     const char * ptr;
@@ -272,7 +294,7 @@ public:
       len = Rf_xlength(obj);
       break;
     case rstring_type::OTHER_ALT_REP:
-      ALTVEC_DATAPTR(obj);
+      DATAPTR_RO(obj); // materialize
       dptr = R_altrep_data2(obj);
       len = Rf_xlength(reinterpret_cast<SEXP>(dptr));
       break;
